@@ -43,16 +43,21 @@ class MyPlots
         t.def_figure("Log_Reds") { log_reds }
         t.def_figure("Side_by_Side") { side_by_side }
         t.def_figure("Two_Ys") { two_yaxes }
+        t.def_figure("Three_Ys") { three_yaxes }
         t.def_figure("Legends") { legends }
         t.def_figure("Two_Ys_with_Legends") { two_yaxes_with_legends }
         t.def_figure("Reds_and_Blues") { reds_blues }
         t.def_figure("Legend_Inside") { legend_inside }
         t.def_figure("Legend_Outside") { legend_outside }
+        t.def_figure("Legend_Inside_with_Background") { legend_inside_with_background }
+        t.def_figure("Legend_Inside_with_Background_using_legend_background") 
+            { legend_inside_with_background_using_legend_background }
         t.def_figure("Inset") { inset }
         t.def_figure("French_Decimal_Separator") { french_decimal_separator }
         t.def_figure("Column_Triplets") { column_triplets }
         t.def_figure("Row_Triplets") { row_triplets }
-        t.def_figure("Rows") { rows }
+        t.def_figure("Two_Rows") { two_rows }
+        t.def_figure("Three_Rows") { rows }
         t.def_figure("Columns") { columns }
         t.def_figure("Array") { array }
         t.def_figure("Trio") { trio }
@@ -70,10 +75,18 @@ class MyPlots
         t.def_figure("Steps") { steps }
         t.def_figure("Splines") { splines }
         t.def_figure("Sampled_Data") { sampled_data }
+        t.def_figure("Sampled_with_grid") { sampled_with_grid }
         t.def_figure("Contours") { samples_with_contours }
 
         t.def_figure("Greens_clipped") { greens_clipped }
 
+        t.def_figure("Blues_far_off") { blues_far_off }
+        t.def_figure("Blues_far_off_vincent") { 
+          t.vincent_or_bill = true
+          blues_far_off 
+        }
+
+
 
         # Starting to play with axes specifications
         t.def_figure("Axes_fun") { axes_fun }
@@ -81,6 +94,10 @@ class MyPlots
         t.model_number = -1
         
         t.def_enter_page_function { enter_page }
+
+        # Uncomment the followiing line if you want tioga to leave
+        # its temporary files behind.
+        t.autocleanup = false
             
     end
     
@@ -151,7 +168,11 @@ EOD
         return [ left_boundary, right_boundary, top_boundary, bottom_boundary ]
     end
     
-    def blues
+    def blues(opts={})
+        opts = {
+          'major_grid' => false,
+          'minor_grid' => false,
+        }.merge(opts)
         read_data
         t.title_color = Crimson
         t.do_box_labels('Blues Plot', 'Position', '\textcolor[rgb]{0,0,1}{Blues}')
@@ -159,6 +180,31 @@ EOD
         xs = @positions
         ys = @blues
         t.show_plot(plot_boundaries(xs,ys,@margin,-1,1)) { 
+            t.show_grid(opts)
+            t.show_polyline(xs,ys,Blue) }
+    end
+
+    def blues_far_off(opts={})
+        opts = {
+          'major_grid' => false,
+          'minor_grid' => false,
+        }.merge(opts)
+        read_data
+        t.title_color = Crimson
+        t.do_box_labels('(shifted) Blues Plot ' + if t.vincent_or_bill
+                                                    '(Vincent)'
+                                                  else
+                                                    '(Bill)'
+                                                  end,
+                        'Position', '\textcolor[rgb]{0,0,1}{Blues}')
+        show_model_number
+        xs = @positions
+        ys = @blues
+        xs *= 0.1
+        xs += 100
+        ys *= 6
+        t.show_plot(plot_boundaries(xs,ys,@margin,-6,6)) { 
+            t.show_grid(opts)
             t.show_polyline(xs,ys,Blue) }
     end
 
@@ -203,20 +249,7 @@ EOD
     end
     
     def blues_with_grid
-        read_data
-        t.do_box_labels('Blues Plot', 'Position', '\textcolor[rgb]{0,0,1}{Blues}')
-        show_model_number
-        xs = @positions
-        ys = @blues
-        t.show_plot(plot_boundaries(xs,ys,@margin,-1,1)) {
-            t.context {
-               t.line_width = 0.5
-               t.line_type = Line_Type_Dot
-               t.line_color = Gray
-               [0,2,4,6].each {|x| t.stroke_line(x,t.bounds_top,x,t.bounds_bottom) }
-               [-1,-0.5,0,0.5,1].each {|y| t.stroke_line(t.bounds_left,y,t.bounds_right,y) }
-            }
-            t.show_polyline(xs,ys,Blue) }
+        blues('major_grid' => true)
     end
     
     def reds
@@ -277,7 +310,7 @@ EOD
         boundaries = setup_lines(@positions, [@blues, @reds], -1, 1)
         xs = @positions
         t.show_plot(boundaries) do
-            t.show_polyline(xs,@blues,Blue,'Blues')
+            t.show_polyline(xs,@blues,Blue,'really Blues')
             t.show_polyline(xs,@reds,Red,'Reds')
         end
         show_model_number
@@ -298,7 +331,7 @@ EOD
     
     def two_yaxes
         read_data
-        t.do_box_labels('Same X, Different Y\'s', 'Position', nil)
+        t.do_box_labels('Same X, 2 Different Y\'s', 'Position', nil)
         t.subplot {
             t.yaxis_loc = t.ylabel_side = LEFT;
             t.right_edge_type = AXIS_HIDDEN; reds }
@@ -307,23 +340,68 @@ EOD
             t.left_edge_type = AXIS_HIDDEN; greens }
         show_model_number
     end
+    
+    def three_yaxes
+        read_data
+        t.rescale(0.75)
+        t.ylabel_shift = 1.4
+        t.set_subframe('right_margin' => 0.3, 'top_margin' => 0.1, 'bottom_margin' => 0.1)
+        t.do_box_labels('Same X, 3 Different Y\'s', 'Position', nil)
+        t.subplot {
+            t.yaxis_loc = t.ylabel_side = LEFT;
+            t.right_edge_type = AXIS_HIDDEN
+            reds 
+        }
+        t.subplot {
+            t.yaxis_loc = t.ylabel_side = RIGHT;
+            t.left_edge_type = AXIS_HIDDEN
+            greens
+        }
+
+        t.context do 
+            xs = @positions
+            ys = @blues
+            bounds = plot_boundaries(xs,ys.mul(2),@margin,-1,1)
+            left_boundary = bounds[0]
+            right_boundary = bounds[1]
+            top_boundary = bounds[2]
+            bottom_boundary = bounds[3]
+            dx = right_boundary - left_boundary
+            t.set_bounds(bounds)
+            t.show_polyline(xs,ys,Blue)
+            spec = {
+                'ticks_outside' => false,
+                'ticks_inside' => true,
+                'from' => [right_boundary + 0.3*dx, top_boundary],
+                'to' => [right_boundary + 0.3*dx, bottom_boundary],
+            }
+            t.show_axis(spec)
+            t.yaxis_loc = t.ylabel_side = RIGHT
+            t.ylabel_shift = 5
+            t.show_ylabel('\textcolor[rgb]{0,0,1}{Blues}')
+        end
+        show_model_number
+    end
 
 
     def two_yaxes_with_legends
       read_data
       t.do_box_labels('Same X, Different Y\'s', 'Position', nil)
+      t.legend_text_dy = 1.2
       t.show_plot_with_legend('legend_left_margin' => 0.65,
                               'plot_scale' => 1,
                               'legend_scale' => 1.3,
                               'plot_right_margin' => 0) { 
-        t.subplot {
+        # NOTE: use subfigure rather than subplot to get combined legend info
+        # subplot would be correct if each had its own legend
+        t.subfigure {
           t.yaxis_loc = t.ylabel_side = LEFT;
           t.right_edge_type = AXIS_HIDDEN; 
           t.show_plot(plot_boundaries(@positions,@blues,@margin,-1,1)) {
             t.show_polyline(@positions,@blues,Blue,'Blues')
           }
         }
-        t.subplot {
+        t.subfigure {
           t.yaxis_loc = t.ylabel_side = RIGHT;
           t.left_edge_type = AXIS_HIDDEN; 
           t.show_plot(plot_boundaries(@positions,@reds,@margin,-1,1)) {
@@ -338,11 +416,80 @@ EOD
         read_data
         show_model_number
         t.show_plot_with_legend(
-            'legend_left_margin' => 0.65,
+            'legend_top_margin' => 0.60,
+            'legend_left_margin' => 0.05,
             'plot_scale' => 1,
             'legend_scale' => 1.3,
+            'legend_background_function' => lambda { |bnds| do_legend_background(bnds) },
             'plot_right_margin' => 0) { reds_blues }
     end
+
+
+    def legend_inside_with_background
+        read_data
+        show_model_number
+        t.show_plot_with_legend(
+            'legend_left_margin' => 0.5,
+            'legend_top_margin' => 0.3,
+            'plot_scale' => 1,
+            'legend_scale' => 1.3,
+            'plot_right_margin' => 0) { 
+        # First, the plot:
+        reds_blues 
+        
+        # Then, a background rectangle for the legend
+        t.context do 
+          t.fill_color = White
+          t.fill_opacity = 0.7
+          t.fill_rect(3.2,-0.2, 2.3, 0.5)
+        end
+      }
+    end
+
+    def legend_inside_with_background_using_legend_background
+        read_data
+        show_model_number
+        t.measure_legends = true
+        t.show_plot_with_legend(
+            'legend_left_margin' => 0.5,
+            'legend_top_margin' => 0.3,
+            'plot_scale' => 1,
+            'legend_scale' => 1.3,
+            'legend_background_function' => lambda { 
+                                  |bnds| do_real_legend_background(bnds) },
+            'plot_right_margin' => 0) { 
+        reds_blues 
+      }
+    end
+
+    
+      
+    def do_legend_background(bnds)
+        t.fill_color = Teal
+        t.fill_opacity = 0.4
+        left = bnds[0]
+        right = 0.35 #bnds[1]
+        top = 0.85 # bnds[2]
+        bot = 0.25 #bnds[3]
+        t.fill_rounded_rect(left, bot, right-left, top-bot, 0.03, 0.03)
+        t.stroke_color = Black
+        t.stroke_width = 0.7
+        t.stroke_rounded_rect(left, bot, right-left, top-bot, 0.03, 0.03)
+    end 
+
+    def do_real_legend_background(bnds)
+        t.fill_color = Teal
+        t.fill_opacity = 0.4
+        left = bnds[0]
+        right = bnds[1]
+        top =  bnds[2]
+        bot = bnds[3]
+        t.fill_rounded_rect(left, bot, right-left, top-bot, 0.03, 0.03)
+        t.stroke_color = Black
+        t.stroke_width = 0.7
+        t.stroke_rounded_rect(left, bot, right-left, top-bot, 0.03, 0.03)
+    end 
+
     
     def legend_outside
         read_data
@@ -434,6 +581,23 @@ EOD
         end
     end
     
+    def two_rows
+        read_data
+        t.landscape
+        show_model_number
+        t.do_box_labels('Blues, Reds, Greens', 'Position', nil)
+        t.rescale(0.8)
+        num_plots = 2
+        t.subplot(t.row_margins('num_rows' => num_plots, 'row' => 1)) do
+            t.xaxis_type = AXIS_WITH_TICKS_ONLY
+            blues
+        end
+        t.subplot(t.row_margins('num_rows' => num_plots, 'row' => 2)) do 
+            t.top_edge_type = AXIS_HIDDEN
+            greens
+        end
+    end
+    
     def columns
         read_data
         t.set_aspect_ratio(2)
@@ -737,6 +901,7 @@ EOD
         t.do_box_labels("Splines", "Position", "Average Count")
         xs = Dvector[ 1.0, 2.0, 5.0, 6.0, 7.0, 8.0, 10.0, 13.0, 17.0 ]
         ys = Dvector[ 3.0, 3.7, 3.9, 4.2, 5.7, 6.6,  7.1,  6.7,  4.5 ]
+        scale = Dvector[ 0.6, 0.9 ]
         t.show_plot([-1, 19, 8, 2]) do
             t.fill_color = FloralWhite
             t.fill_frame
@@ -747,8 +912,8 @@ EOD
             t.append_interpolant_to_path(interp)
             t.stroke_color = Black
             t.stroke
-            t.show_marker('Xs' => xs, 'Ys' => ys, 'marker' => Bullet, 
-                'scale' => 0.6, 'color' => Red);
+            t.show_marker('Xs' => xs, 'Ys' => ys, 'marker' => [ Bullet, Star ], 
+                          'scale' => scale, 'color' => [ Red, Blue, Teal ]);
         end
     end
     
@@ -771,6 +936,29 @@ EOD
                 'w' => @eos_data_xlen, 'h' => @eos_data_ylen)
         end
     end
+        
+    def sampled_image_with_grid(title, colormap = nil)
+        t.do_box_labels(title, 'Log Density', 'Log Temperature')
+        data = get_press_image
+        xs = @eos_logRHOs
+        ys = @eos_logTs
+        colormap = t.mellow_colormap if colormap == nil
+        t.show_plot([@eos_xmin, @eos_xmax, @eos_ymax, @eos_ymin]) do
+            t.fill_color = Wheat
+            t.fill_frame
+            t.context do
+                clip_press_image
+                t.show_image(
+                    'll' => [xs.min, ys.min], 
+                    'lr' => [xs.max, ys.min], 
+                    'ul' => [xs.min, ys.max], 
+                    'color_space' => colormap, 
+                    'data' => data, 'value_mask' => 255,
+                    'w' => @eos_data_xlen, 'h' => @eos_data_ylen)
+            end
+            t.show_grid('stroke_color' => Teal, 'rescale_lines' => 0.5, 'major_grid' => true)
+        end
+    end
     
     def color_bar(ylabel, levels = nil)
         xmin = 0; xmax = 1; xmid = 0.5
@@ -806,6 +994,15 @@ EOD
             'bottom_margin' => 0.05) { color_bar(title) }
     end
     
+    def sampled_with_grid
+        t.rescale(0.8)
+        title = 'Log Opacity'
+        t.subplot('right_margin' => @image_right_margin) { sampled_image_with_grid(title) }
+        t.subplot('left_margin' => 0.95, 
+            'top_margin' => 0.05, 
+            'bottom_margin' => 0.05) { color_bar(title) }
+    end
+    
     def read_press_image_data
         return unless @opacity_data == nil
         @eos_xmin = -8.5; @eos_xmax = 2.5
