@@ -230,6 +230,10 @@ FM *Get_FM(OBJ_PTR fmkr, int *ierr) {
    RO_BOOL_ATTR(left_edge_visible)
    INT_ATTR(right_edge_type)
    RO_BOOL_ATTR(right_edge_visible)
+
+/* Major ticks picking algorithm */
+   BOOL_ATTR(vincent_or_bill)
+
     
 /* Legend */
    DBL_ATTR(legend_text_width)
@@ -382,6 +386,8 @@ void Init_FigureMaker(void) {
 /* colors */
    rb_define_method(cFM, "hls_to_rgb", FM_hls_to_rgb, 1);
    rb_define_method(cFM, "rgb_to_hls", FM_rgb_to_hls, 1);
+   rb_define_method(cFM, "string_hls_to_rgb!", FM_string_hls_to_rgb_bang, 1);
+   rb_define_method(cFM, "string_rgb_to_hls!", FM_string_rgb_to_hls_bang, 1);
 /* coordinate system conversions */
    rb_define_method(cFM, "convert_inches_to_output", FM_convert_inches_to_output, 1);
    rb_define_method(cFM, "convert_output_to_inches", FM_convert_output_to_inches, 1);
@@ -487,12 +493,13 @@ void Init_FigureMaker(void) {
    rb_define_method(cFM, "private_radial_shading", FM_private_radial_shading, 13);
 /* markers */
    rb_define_method(cFM, "register_font", FM_register_font, 1);
-   rb_define_method(cFM, "private_show_marker", FM_private_show_marker, 15);
+   rb_define_method(cFM, "private_show_marker", FM_private_show_marker, 1);
    rb_define_method(cFM, "marker_string_info", FM_marker_string_info, 3);
 /* images */
    rb_define_method(cFM, "private_show_jpg", FM_private_show_jpg, 5);
    rb_define_method(cFM, "private_show_rgb_image", FM_private_show_rgb_image, 11);
-   rb_define_method(cFM, "private_show_cmyk_image", FM_private_show_rgb_image, 11);
+   rb_define_method(cFM, "private_show_hls_image", FM_private_show_hls_image, 11);
+   rb_define_method(cFM, "private_show_cmyk_image", FM_private_show_cmyk_image, 11);
    rb_define_method(cFM, "private_show_grayscale_image", FM_private_show_grayscale_image, 11);
    rb_define_method(cFM, "private_show_monochrome_image", FM_private_show_monochrome_image, 12);
    rb_define_method(cFM, "private_show_image", FM_private_show_image, 15);
@@ -617,6 +624,10 @@ void Init_FigureMaker(void) {
    attr_reader(left_edge_visible)
    attr_accessors(right_edge_type)
    attr_reader(right_edge_visible)
+
+
+/* Major ticks algorithm */
+   attr_accessors(vincent_or_bill)
     
 /* Legend */
    attr_accessors(legend_text_width)
@@ -644,6 +655,9 @@ void Init_FigureMaker(void) {
    RB_IMPORT_SYMBOL(cDvector, Dvector_Create);
    RB_IMPORT_SYMBOL(cDvector, Dvector_Data_Resize);
    RB_IMPORT_SYMBOL(cDvector, Dvector_Data_Replace);
+   RB_IMPORT_SYMBOL(cDvector, isa_Dvector);
+   RB_IMPORT_SYMBOL(cDvector, len_Dvector);
+   RB_IMPORT_SYMBOL(cDvector, access_Dvector);
    RB_IMPORT_SYMBOL(cDvector, Dvector_Data_for_Read);
    RB_IMPORT_SYMBOL(cDvector, Dvector_Data_for_Write);
    RB_IMPORT_SYMBOL(cDvector, Dvector_Store_Double);
@@ -665,6 +679,9 @@ void Init_FigureMaker(void) {
 IMPLEMENT_SYMBOL(Dvector_Create);
 IMPLEMENT_SYMBOL(Dvector_Data_Resize);
 IMPLEMENT_SYMBOL(Dvector_Data_Replace);
+IMPLEMENT_SYMBOL(isa_Dvector);
+IMPLEMENT_SYMBOL(len_Dvector);
+IMPLEMENT_SYMBOL(access_Dvector);
 IMPLEMENT_SYMBOL(Dvector_Data_for_Read);
 IMPLEMENT_SYMBOL(Dvector_Data_for_Write);
 IMPLEMENT_SYMBOL(Dvector_Store_Double);
