@@ -281,7 +281,10 @@ typedef struct {
     bool left_edge_visible;
     int right_edge_type;
     bool right_edge_visible;
-    
+
+    /* General axis stuff*/
+    bool vincent_or_bill;
+
     /* Legend */
     // units are text heights for x and y for locations and sizes
     double legend_line_x0; // x location of start of line
@@ -454,6 +457,8 @@ extern void c_xaxis_stroke_color_set(OBJ_PTR fmkr, FM *p, OBJ_PTR valc_hls_to_rg
 extern OBJ_PTR c_xaxis_stroke_color_get(OBJ_PTR fmkr, FM *pc_hls_to_rgb, int *ierr); // value is array of [r, g, b] intensities from 0 to 1
 extern void c_yaxis_stroke_color_set(OBJ_PTR fmkr, FM *p, OBJ_PTR valc_hls_to_rgb, int *ierr);
 extern OBJ_PTR c_yaxis_stroke_color_get(OBJ_PTR fmkr, FM *pc_hls_to_rgb, int *ierr); // value is array of [r, g, b] intensities from 0 to 1
+extern void c_string_hls_to_rgb_bang(OBJ_PTR fmkr, FM *p, unsigned char* hls_str, long len, int *ierr);
+extern void c_string_rgb_to_hls_bang(OBJ_PTR fmkr, FM *p, unsigned char* rgb_str, long len, int *ierr);
 
 /*======================================================================*/
 // pdfcoords.c
@@ -643,11 +648,7 @@ extern void Write_Font_Widths(void);
 
 extern OBJ_PTR c_register_font(OBJ_PTR fmkr, FM *p, char *font_name, int *ierr);
 extern OBJ_PTR c_marker_string_info(OBJ_PTR fmkr, FM *p, int fnt, unsigned char *text, double scale, int *ierr);
-extern void c_private_show_marker(
-   OBJ_PTR fmkr, FM *p, int int_args, OBJ_PTR stroke_width, OBJ_PTR string,
-   OBJ_PTR x, OBJ_PTR y, OBJ_PTR x_vec, OBJ_PTR y_vec,
-   double h_scale, double v_scale, double scale, double it_angle, double ascent_angle, double angle,
-   OBJ_PTR fill_color, OBJ_PTR stroke_color, int *ierr);
+extern void c_private_show_marker(OBJ_PTR fmkr, FM *p, OBJ_PTR args, int *ierr);
 
 /*======================================================================*/
 // texout.c
