@@ -34,7 +34,7 @@ class FigureMaker
 
     # This URL will contain tioga-(...) when it is exported from the
     # SVN repository. This is where we'll look for version information.
-    SVN_URL = '$HeadURL: svn+ssh://rubyforge.org/var/svn/tioga/tags/tioga/Tioga%201.8/split/Tioga/lib/FigMkr.rb $'
+    SVN_URL = '$HeadURL: svn+ssh://rubyforge.org/var/svn/tioga/tags/tioga/Tioga%201.10/split/Tioga/lib/FigMkr.rb $'
 
     TIOGA_VERSION = if SVN_URL =~ /tags\/tioga\/Tioga%20([^\/]+)/
                        $1
@@ -66,14 +66,28 @@ class FigureMaker
       @@which_pdflatex = s
     end
     
-    
     def FigureMaker.make_name_lookup_hash(ary)
         dict = Hash.new
         ary.each { |name| dict[name] = true }
         return dict
     end
-
-
+    
+    def FigureMaker.make_pdf(name='immediate', &block)
+      self.default.make_pdf(name, &block)
+    end
+    
+    def FigureMaker.def_enter_page_function(&cmd)
+      self.default.def_enter_page_function(&cmd)
+    end
+    
+    def FigureMaker.page_style(&cmd) 
+      FigureMaker.def_enter_page_function(&cmd)
+    end
+    
+    def FigureMaker.exec(&cmd) 
+      cmd.call(self.default)
+    end
+    
 
     attr_accessor :title
     attr_accessor :xlabel
@@ -128,7 +142,7 @@ class FigureMaker
     
     attr_accessor :num_error_lines
 
-    # Whether or not to create +save_dir+ if it doesn't exist
+    # Whether or not to create _save_dir_ if it doesn't exist
     attr_accessor :create_save_dir
 
     # Whether or not do do automatic cleanup of the files
@@ -140,6 +154,13 @@ class FigureMaker
     # An accessor for @measures_info:
     attr_accessor :measures_info
 
+
+
+    # If we want to use #legend_bounding_box. It is off by default
+    # as it causes a systematic second run of pdflatex.
+    attr_accessor :measure_legends
+
+
     # old preview attributes -- to be removed later
     
             attr_accessor :model_number
@@ -229,6 +250,7 @@ class FigureMaker
             'plot_left_margin' => 0.0,
             'plot_right_margin' => 0.18,
             'plot_scale' => 1,
+            'legend_background_function' => false,
             'legend_scale' => 1 }
         
         @marker_defaults = { 
@@ -279,6 +301,12 @@ class FigureMaker
 
         # We *must* initialize the measures_info hash.
         @measures_info = {}
+
+        # We don't measure legends by default.
+        @measure_legends = false
+
+        # By default, we use Bill's algorithm for major ticks positions
+        self.vincent_or_bill = false
     end
     
 
@@ -611,6 +639,7 @@ class FigureMaker
       'marker_scale' => nil,
     }
 
+
     def save_legend_info(arg)
         if arg.kind_of?String
             dict = { 'text' => arg }
@@ -625,8 +654,9 @@ class FigureMaker
         end
         @legend_info << dict
     end
+    
 
-    def show_legend
+    def show_legend(legend_background_function=nil)
         char_dx = self.default_text_height_dx
         char_dy = self.default_text_height_dy
         line_ht_x = char_dx * self.legend_scale
@@ -641,46 +671,103 @@ class FigureMaker
             end
         end
         xright = x + ltw*line_ht_x
+
         y = 1.0 - self.legend_text_ystart*line_ht_y
         update_bbox(xright, y)
-        dy = -self.legend_text_dy*line_ht_y
         line_x0 = self.legend_line_x0*line_ht_x
         line_x1 = self.legend_line_x1*line_ht_x
         line_dy = self.legend_line_dy*line_ht_y
-        self.label_left_margin = self.label_right_margin = self.label_top_margin = self.label_bottom_margin = -1e99
-        @legend_info.each do |dict|
-            text = dict['text']
-            if text != nil
-                show_text('text' => text,
-                    'x' => x, 'y' => y, 'scale' => self.legend_scale,
-                    'justification' => self.legend_justification,
-                    'alignment' => self.legend_alignment)
-            end
-            line_width = dict['line_width']
-            line_type = dict['line_type']
-            unless (line_width < 0) || ((line_type.kind_of?String) && (line_type.casecmp('none') == 0))
-                self.line_color = dict['line_color']
-                self.line_width = dict['line_width']
-                self.line_cap = dict['line_cap']
-                self.line_type = line_type
-                stroke_line(line_x0, y+line_dy, line_x1, y+line_dy)
-            end
-            # place any marker right in the middle of the line
-            if dict['marker_dict'] != nil
-                marker_dict = dict['marker_dict']
-                marker_dict['x'] = 0.5*(line_x0 + line_x1)
-                marker_dict['y'] = y+line_dy
-                show_marker(marker_dict)
-            elsif dict['marker']
-                show_marker( 'x' => 0.5*(line_x0 + line_x1),
-                        'y' => (y+line_dy),
-                        'marker' => dict['marker'],
-                        'color' => dict['marker_color'],
-                        'scale' => dict['marker_scale'])
-            end
-            dy = dict['dy']; dy = 1 if dy == nil
-            y -= line_ht_y * dy
+
+        # If we are measuring legends, we come up with a more
+        # accurate version of xright:
+        if @measure_legends
+          xright = x + convert_output_to_figure_dx(legend_text_width)
+          xright += line_x0     # To leave a symetric space around
+          # the legend !
         end
+
+        self.label_left_margin = self.label_right_margin = self.label_top_margin = self.label_bottom_margin = -1e99
+        unless legend_background_function == nil || legend_background_function == false
+           ybot = y
+          # we need to remove the first element ;-)...
+           @legend_info[1..-1].each do |dict|
+                dy = dict['dy']; dy = 1 if dy == nil
+                ybot -= line_ht_y * dy
+           end
+           # We add half a line below to look good
+           ybot -= line_ht_y * 0.5
+           ytop = y + @legend_info.first['dy'] * line_ht_y * 0.5
+           legend_background_function.call([ 0, xright, ytop, ybot ])
+        end
+        legend_index = 0
+        if @legend_info != nil
+            @legend_info.each do |dict|
+                text = dict['text']
+                if text != nil
+                    # We prepare a dictionnary:
+                    dct = { 'text' => text,
+                             'x' => x, 'y' => y, 'scale' => self.legend_scale,
+                             'justification' => self.legend_justification,
+                             'alignment' => self.legend_alignment }
+                    if @measure_legends
+                      dct['measure'] = "legend-#{legend_index}"
+                      legend_index += 1
+                    end
+                    show_text(dct)
+                end
+                line_width = dict['line_width']
+                line_type = dict['line_type']
+                unless (line_width < 0) || ((line_type.kind_of?String) && (line_type.casecmp('none') == 0))
+                
+                    save_line_color = self.line_color
+                    save_line_width = self.line_width
+                    save_line_cap = self.line_cap
+                    save_line_type = self.line_type
+                    
+                    self.line_color = dict['line_color']
+                    self.line_width = dict['line_width']
+                    self.line_cap = dict['line_cap']
+                    self.line_type = line_type
+                    
+                    stroke_line(line_x0, y+line_dy, line_x1, y+line_dy)
+                
+                    self.line_color = save_line_color
+                    self.line_width = save_line_width
+                    self.line_cap = save_line_cap
+                    self.line_type = save_line_type
+                    
+                end
+                # place any marker right in the middle of the line
+                if dict['marker_dict'] != nil
+                    marker_dict = dict['marker_dict']
+                    marker_dict['x'] = 0.5*(line_x0 + line_x1)
+                    marker_dict['y'] = y+line_dy
+                    show_marker(marker_dict)
+                elsif dict['marker']
+                    show_marker( 'x' => 0.5*(line_x0 + line_x1),
+                            'y' => (y+line_dy),
+                            'marker' => dict['marker'],
+                            'color' => dict['marker_color'],
+                            'scale' => dict['marker_scale'])
+                end
+                dy = dict['dy']; dy = 1 if dy == nil
+                y -= line_ht_y * dy
+            end
+        end
+    end
+
+    # Returns the exact length of the text width in output
+    # coordinates
+    def legend_text_width
+      index = 0
+      width = 0.0
+      while h = get_text_size("legend-#{index}") and h.key?('width')
+        if width < h['width'] 
+          width = h['width'] 
+        end
+        index += 1
+      end
+      return 10 * width
     end
 
     def legend_height
@@ -808,17 +895,17 @@ class FigureMaker
         
         unless entry_function == nil 
             begin
-                entry_function.call
+                entry_function.call(self)
             rescue Exception => er
                 report_error(er, nil)
             end
         end
         
-        result = cmd.call
+        result = cmd.call(self)
         
         unless exit_function == nil 
             begin
-                exit_function.call
+                exit_function.call(self)
             rescue Exception => er
                 report_error(er, nil)
             end
@@ -839,7 +926,7 @@ class FigureMaker
             end
         end
         
-        result = cmd.call
+        result = cmd.call(self)
         
         unless exit_function == nil 
             begin
@@ -857,14 +944,24 @@ class FigureMaker
     def show_plot(bounds=nil,&cmd)       
         trace_cmd_one_arg(@enter_show_plot_function, @exit_show_plot_function, bounds) {        
             set_bounds(bounds)
-            context { clip_to_frame; cmd.call }
+            context { clip_to_frame; cmd.call(self) }
             show_plot_box
         }      
     end
     
+    
+    def show_plot_without_clipping(bounds=nil,&cmd)       
+        trace_cmd_one_arg(@enter_show_plot_function, @exit_show_plot_function, bounds) {        
+            set_bounds(bounds)
+            context { cmd.call(self) }
+            show_plot_box
+        }      
+    end
+    
+    
     def subfigure(margins=nil,&cmd)
         trace_cmd_one_arg(@enter_subfigure_function, @exit_subfigure_function, margins) {        
-            context { doing_subfigure; set_subframe(margins); cmd.call }
+            context { doing_subfigure; set_subframe(margins); cmd.call(self) }
         }      
     end
     
@@ -879,7 +976,7 @@ class FigureMaker
     def subplot(margins=nil,&cmd)
         trace_cmd_one_arg(@enter_subplot_function, @exit_subplot_function, margins) {        
             reset_legend_info
-            context { doing_subplot; set_subframe(margins); cmd.call }
+            context { doing_subplot; set_subframe(margins); cmd.call(self) }
         }      
     end
     
@@ -943,7 +1040,6 @@ class FigureMaker
 
     def context(&cmd)
         trace_cmd_no_arg(@enter_context_function, @exit_context_function) {       
-             
            save_title = self.title
            save_xlabel = self.xlabel
            save_ylabel = self.ylabel
@@ -957,7 +1053,7 @@ class FigureMaker
            save_fm_data = Dvector.new(@@fm_data_size).replace(@fm_data)
            pdf_gsave
            begin
-               cmd.call       
+               cmd.call(self)       
            ensure
                pdf_grestore
                self.title = save_title
@@ -971,8 +1067,7 @@ class FigureMaker
                self.yaxis_locations_for_minor_ticks = save_yaxis_locations_for_minor_ticks
                self.yaxis_tick_labels = save_yaxis_tick_labels
                @fm_data.replace(save_fm_data)            
-           end
-            
+           end            
         }      
     end
 
@@ -1066,10 +1161,92 @@ class FigureMaker
         end
     end
     
+    @@keys_for_show_grid = %w[
+      stroke_color line_type rescale_lines line_width stroke_opacity stroke_transparency
+    ]
+
+    # Draws the gridlines associated with the X and Y axis.
+    #
+    # The following list shows the keys that cen be used in +dict+.  In the
+    # list, 'A' stands for either 'x' or 'y', and 'M' stands for 'major' or
+    # 'minor'.  Keys can also be specified without the "A_M_" prefix to specify
+    # "grid-global" defaults.  The +grid+ key, if true, enables both major and
+    # minor gridlines for both x and y axes.
+    #
+    # * A_M_grid
+    # * A_M_line_type
+    # * A_M_line_width
+    # * A_M_stroke_opacity
+    # * A_M_stroke_transparency
+    # * A_M_stroke_color
+    def show_grid(dict={})
+      dict = {
+        'line_type' => Line_Type_Dot,
+        'rescale_lines' => 0.25,
+        'stroke_color' => Gray,
+        'major_grid' => true,
+        'minor_grid' => false,
+        'x_minor_rescale_lines' => 0.5,
+        'y_minor_rescale_lines' => 0.5,
+      }.merge(dict)
+
+      # "grid-global" context
+      context do
+        @@keys_for_show_grid.each do |k|
+          next unless val = dict[k]
+          k += '=' if respond_to? "#{k}="
+          next unless respond_to? k
+          send(k, val)
+        end
+
+        if dict['x_major_grid'] || dict['x_minor_grid'] ||
+           dict[  'major_grid'] || dict[  'minor_grid'] || dict['grid']
+          top = axis_information(TOP)
+          bottom = axis_information(BOTTOM)
+          y0 = bottom['y0']
+          y1 = top['y1']
+          # Do minor and major grid lines in same context so minor lines will
+          # inherit properties from major lines.
+          context do
+            %w[ major minor ].each do |m|
+              next unless dict["x_#{m}_grid"] || dict["#{m}_grid"] || dict["grid"]
+              @@keys_for_show_grid.each do |k|
+                next unless val = dict["x_#{m}_#{k}"]
+                k += '=' if respond_to? "#{k}="
+                send(k, val)
+              end
+              bottom[m].each {|x| stroke_line(x, y0, x, y1)}
+            end
+          end
+        end
+
+        if dict['y_major_grid'] || dict['y_minor_grid'] ||
+           dict[  'major_grid'] || dict[  'minor_grid'] || dict['grid']
+          left = axis_information(LEFT)
+          right = axis_information(RIGHT)
+          x0 = left['x0']
+          x1 = right['x1']
+          # Do minor and major grid lines in same context so minor lines will
+          # inherit properties from major lines.
+          context do
+            %w[ major minor ].each do |m|
+              next unless dict["y_#{m}_grid"] || dict["#{m}_grid"] || dict["grid"]
+              @@keys_for_show_grid.each do |k|
+                next unless val = dict["y_#{m}_#{k}"]
+                k += '=' if respond_to? "#{k}="
+                send(k, val)
+              end
+              left[m].each {|y| stroke_line(x0, y, x1, y)}
+            end
+          end
+        end
+      end
+    end
+
     @@keys_for_show_plot_with_legend = FigureMaker.make_name_lookup_hash([
         'legend_top_margin', 'legend_bottom_margin', 'legend_left_margin', 'legend_right_margin',
         'plot_top_margin', 'plot_bottom_margin', 'plot_left_margin', 'plot_right_margin',
-        'plot_scale', 'legend_scale' ])
+        'plot_scale', 'legend_scale', 'legend_background_function' ])
     def show_plot_with_legend(dict=nil, &cmd)
         check_dict(dict, @@keys_for_show_plot_with_legend, 'show_plot_with_legend') if dict != nil
         legend_top_margin = get_if_given_else_use_default_dict(dict, 'legend_top_margin', @legend_defaults)
@@ -1082,13 +1259,18 @@ class FigureMaker
         plot_left_margin = get_if_given_else_use_default_dict(dict, 'plot_left_margin', @legend_defaults)
         plot_right_margin = get_if_given_else_use_default_dict(dict, 'plot_right_margin', @legend_defaults)
         plot_scale = get_if_given_else_use_default_dict(dict, 'plot_scale', @legend_defaults)
+        legend_background_function = get_if_given_else_default(dict, 'legend_background_function', nil)
+        @legend_subframe = [legend_left_margin, legend_right_margin, 
+                            legend_top_margin, legend_bottom_margin]
         reset_legend_info
         rescale(plot_scale)
-        subplot([plot_left_margin, plot_right_margin, plot_top_margin, plot_bottom_margin]) { cmd.call }
-        set_subframe([legend_left_margin, legend_right_margin, legend_top_margin, legend_bottom_margin])
+        subplot([plot_left_margin, plot_right_margin, plot_top_margin, plot_bottom_margin]) { cmd.call(self) }
+        # We temporary store the legend information in @legend_subframes
+        set_subframe(@legend_subframe)
         rescale(legend_scale) # note that legend_scale is an addition to the plot_scale, not a replacement
         @pr_margin = plot_right_margin
-        show_legend
+        show_legend(legend_background_function)
+        @legend_subframe = nil
     end
     
     def append_points_with_gaps_to_path(xs, ys, gaps, close_subpaths = false)
@@ -1439,6 +1621,17 @@ class FigureMaker
             extend_start, extend_end)
     end
     
+    
+    def string_hls_to_rgb(str)
+      string_hls_to_rgb!(String.new(str))
+    end
+    
+    
+    def string_rgb_to_hls(str)
+      string_rgb_to_hls!(String.new(str))
+    end
+    
+    
     @@keys_for_show_image = FigureMaker.make_name_lookup_hash([
         'll', 'lr', 'ul', 'w', 'width', 'height', 'h',
         'opacity_mask', 'stencil_mask', 
@@ -1624,27 +1817,7 @@ class FigureMaker
         if (marker != nil && string != nil)
             raise "Sorry: Must give either 'marker' or 'string' for show_marker, but not both"
         end
-        if (marker == nil)
-            glyph = stroke_width = nil
-        else
-            if !(marker.kind_of?Array) && marker.size >= 2 && marker.size <= 3
-                raise "Sorry: 'marker' for show_marker must be array of [font_number, char_code] or [font_number, char_code, rendering_mode]"
-            end
-            font = marker[0]
-            glyph = marker[1]
-            if marker.size == 3
-                mode = STROKE if mode == nil
-                stroke_width = marker[2]
-            else
-                mode = FILL if mode == nil
-                stroke_width = nil
-            end
-        end
-        mode = FILL if mode == nil
-        font = Times_Roman if (font == nil && string != nil)
-        if (font != nil && !(font.kind_of? Integer))
-            raise "Sorry: 'font' for show_marker must be an integer font number (see FigureConstants list)"
-        end
+        
         stroke_width = get_if_given_else_default(dict, 'stroke_width', stroke_width)
         angle = get_if_given_else_use_default_dict(dict, 'angle', @marker_defaults)
         scale = get_if_given_else_use_default_dict(dict, 'scale', @marker_defaults)
@@ -1655,10 +1828,11 @@ class FigureMaker
         it_angle = get_if_given_else_use_default_dict(dict, 'italic_angle', @marker_defaults)
         ascent_angle = get_if_given_else_use_default_dict(dict, 'ascent_angle', @marker_defaults)
         glyph = 0 if glyph == nil
-        int_args = glyph*100000 + font*1000 + mode*100 + align*10 + (just+1) # min value for just is -1 
-            # Ruby limits us to 15 args, so pack some small integers together                
-        private_show_marker(int_args, stroke_width, string, x, y, xs, ys,
-            h_scale, v_scale, scale, it_angle, ascent_angle, angle, fill_color, stroke_color)                
+                     
+        args = [marker, font, mode, align, just, stroke_width, string, x, y, xs, ys,
+                h_scale, v_scale, scale, it_angle, ascent_angle, angle, fill_color, stroke_color]
+        private_show_marker(args)       
+                     
         legend_arg = dict['legend']
         if legend_arg != nil
             if legend_arg.kind_of?Hash
@@ -1784,7 +1958,6 @@ class FigureMaker
     end
     
     
-    
     def def_enter_page_function(&cmd)
         if cmd == nil
             raise "Sorry: must provide a command block for def_enter_page_function"
@@ -1950,7 +2123,8 @@ class FigureMaker
     
     # We wrap the call so that if the keys of @measures
     # did change during the call, we call it again.
-    def make_pdf(num) # returns pdf name if successful, false if failed.
+    def make_pdf(num,&cmd) # returns pdf name if successful, false if failed.
+        def_figure(num,&cmd) if Kernel.block_given?
         old_measure_keys = @measures.keys
         num = get_num_for_pdf(num)
         result = start_making_pdf(num)
@@ -1963,6 +2137,7 @@ class FigureMaker
         end
           
         if @measures.keys != old_measure_keys
+          # we dont need to pass &cmd since it has now been defined
           make_pdf(num)
         end
         return @figure_pdfs[num]
@@ -2259,6 +2434,10 @@ class FigureMaker
             private_show_rgb_image(ll[0], ll[1], lr[0], lr[1], ul[0], ul[1], interpolate, w, h, data, mask_xo_num)
             return self
         end
+        if color_space == 'HLS' || color_space == 'hls'
+            private_show_hls_image(ll[0], ll[1], lr[0], lr[1], ul[0], ul[1], interpolate, w, h, data, mask_xo_num)
+            return self
+        end
         if color_space == 'CMYK' || color_space == 'cmyk'
             private_show_cmyk_image(ll[0], ll[1], lr[0], lr[1], ul[0], ul[1], interpolate, w, h, data, mask_xo_num)
             return self
@@ -2310,7 +2489,7 @@ class FigureMaker
         exit_function = @exit_page_function
         unless entry_function == nil 
             begin
-                entry_result = entry_function.call
+                entry_result = entry_function.call(self)
             rescue Exception => er
                 report_error(er, nil)
             end
@@ -2318,7 +2497,7 @@ class FigureMaker
         result = do_cmd(cmd)       
         unless result == false or exit_function == nil 
             begin
-                exit_result = exit_function.call
+                exit_result = exit_function.call(self)
             rescue Exception => er
                 report_error(er, nil)
             end
@@ -2329,7 +2508,7 @@ class FigureMaker
 
     def do_cmd(cmd)  # the C implementation uses this to call Ruby commands
         begin
-            cmd.call
+            cmd.call(self)
             return true
         rescue Exception => er
             report_error(er, nil)
