@@ -20,9 +20,13 @@
    along with Tioga; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
+#include <math.h>
+
 #include "figures.h"
 #include "pdfs.h"
 
+
+
 /* 
    Here is my (Vincent) big TODO-list for the axes stuff:
    * provide a way to reliably get the location from major/minor
@@ -81,6 +85,9 @@ typedef struct {
    int num_minors; // number of minor intervals
    double interval; // tick interval
    int location;
+   bool vincent_or_bill; 	/* True if we use Vincent's algorithm
+				   for picking major ticks
+				*/
 } PlotAxis;
 
 static void figure_moveto(OBJ_PTR fmkr, FM *p, double x, double y, int *ierr) // figure coords
@@ -147,6 +154,8 @@ static void Get_xaxis_Specs(OBJ_PTR fmkr, FM *p, PlotAxis *s, int *ierr)
    s->numeric_label_justification = p->xaxis_numeric_label_justification;
    s->numeric_label_frequency = p->xaxis_numeric_label_frequency;
    s->numeric_label_phase = p->xaxis_numeric_label_phase;
+
+   s->vincent_or_bill = p->vincent_or_bill;
 }
 
 static void Get_yaxis_Specs(OBJ_PTR fmkr, FM *p, PlotAxis *s, int *ierr)
@@ -184,6 +193,8 @@ static void Get_yaxis_Specs(OBJ_PTR fmkr, FM *p, PlotAxis *s, int *ierr)
    s->numeric_label_justification = p->yaxis_numeric_label_justification;
    s->numeric_label_frequency = p->yaxis_numeric_label_frequency;
    s->numeric_label_phase = p->yaxis_numeric_label_phase;
+
+   s->vincent_or_bill = p->vincent_or_bill;
 }
 
 /*======================================================================*/
@@ -474,6 +485,42 @@ static char **Get_Labels(OBJ_PTR fmkr, FM *p, PlotAxis *s, int *ierr)
    return labels;
 }
          
+static int Pick_Number_of_Minor_Intervals(double length, int *ierr)
+{
+   double t1, fuzz, newlen;
+   int np, num_subintervals, inewlen;
+   if (length < 10.0 && length > 1.0) {
+      newlen = length;
+   } else {
+      t1 = (double) log10(length);
+      np = (int) floor(t1);
+      t1 -= np;
+      newlen = pow(10.0, t1);
+   }
+   inewlen = (int) floor(newlen + 0.5);
+   fuzz = newlen - inewlen;
+   fuzz = abs(fuzz);
+   num_subintervals = 4;
+   if (fuzz < 1e-2) {
+      switch (inewlen) {
+         case 0: case 1: case 2: num_subintervals = 4; break;
+         case 4: break;
+         case 8: case 10: num_subintervals = 2; break;
+         case 3: case 6: case 9: num_subintervals = 3; break;
+         default: num_subintervals = 5;
+      }
+   } else {
+      if (newlen > 5.0) num_subintervals = 4;
+      else if (newlen > 3.0) num_subintervals = 5;
+      else if (newlen > 1.5) num_subintervals = 5;
+   }
+   return num_subintervals;
+}
+
+/* Refactoring of the axis location picking code */
+
+/* First, Bill's point of view */
+
 static double *Pick_Locations_for_Major_Ticks(double interval, 
    double axis_min, double axis_max, int *num_locations, int *ierr)
 {
@@ -503,34 +550,6 @@ static double *Pick_Locations_for_Major_Ticks(double interval,
    return majors;
 }
 
-static int Pick_Number_of_Minor_Intervals(double length, int *ierr)
-{
-   double t1, fuzz, newlen;
-   int np, num_subintervals, inewlen;
-   t1 = (double) log10(length);
-   np = (int) floor(t1);
-   t1 -= np;
-   newlen = pow(10.0, t1);
-   inewlen = (int) floor(newlen + 0.5);
-   fuzz = newlen - inewlen;
-   fuzz = abs(fuzz);
-   num_subintervals = 4;
-   if (fuzz < 1e-2) {
-      switch (inewlen) {
-         case 0: case 1: case 2: num_subintervals = 4; break;
-         case 4: break;
-         case 8: case 10: num_subintervals = 2; break;
-         case 3: case 6: case 9: num_subintervals = 3; break;
-         default: num_subintervals = 5;
-      }
-   } else {
-      if (newlen > 5.0) num_subintervals = 4;
-      else if (newlen > 3.0) num_subintervals = 5;
-      else if (newlen > 1.5) num_subintervals = 5;
-   }
-   return num_subintervals;
-}
-
 static void Pick_Major_Tick_Interval(OBJ_PTR fmkr, FM *p, 
       double tick_min, double tick_gap, double length, bool log_vals, double *tick, int *ierr)
 {
@@ -569,6 +588,81 @@ static void Pick_Major_Tick_Interval(OBJ_PTR fmkr, FM *p,
    }
 }
 
+static double * pick_major_ticks_positions_Bill(OBJ_PTR fmkr, FM *p, 
+						double axis_min, 
+						double axis_max, 
+						int *num_locations, 
+						double tick_min, 
+						double tick_gap, 
+						double length, 
+						bool log_vals, 
+						double *tick,
+						int *ierr)
+{
+   /* This code is using Bill's initial implementation */
+   Pick_Major_Tick_Interval(fmkr, p, tick_min, tick_gap, 
+			    length, log_vals, tick, ierr);
+   /*    printf("Tick gap: %f, length: %f, tick_min: %f, tick interval: %f\n", */
+   /* 	  tick_gap, length, tick_min, *tick); */
+   if (*ierr != 0) 
+      return NULL;
+   return Pick_Locations_for_Major_Ticks(*tick, axis_min, axis_max, 
+					 num_locations, ierr);
+}
+
+/* Then, Vincent's point of view ;-) ... */
+
+static double natural_distances[] = { 1.0, 2.0, /*2.5,*/ 5.0, 10.0 };
+const int nb_natural_distances = sizeof(natural_distances)/sizeof(double);
+
+static double * pick_major_ticks_positions_Vincent(OBJ_PTR fmkr, FM *p, 
+				   double axis_min, 
+				   double axis_max, 
+				   int *num_locations, 
+				   double tick_min, 
+				   double tick_gap, 
+				   double length, 
+				   bool log_vals, 
+				   double *tick,
+				   int *ierr)
+{
+   /* I like Bill's way of handling log scale */
+   if(log_vals)
+      return pick_major_ticks_positions_Bill(fmkr,p, 
+					     axis_min, axis_max, 
+					     num_locations, tick_min, 
+					     tick_gap, length, log_vals, 
+					     tick, ierr);
+   /* The factor by which you need to divide to get
+      the tick_min within [1,10[ */
+   double factor = pow(10, floor(log10(tick_min)));
+   double norm_tick_min = tick_min/factor;
+   int i;
+   
+   /*    printf("axis_min: %g\taxis_max: %g\n", axis_min, axis_max); */
+   /* We get the one just above tick_min */
+   for(i = 0; i < nb_natural_distances; i++)
+      if(natural_distances[i] >= norm_tick_min)
+	 break;
+
+
+   *tick = natural_distances[i] * factor;
+/*    printf("norm_tick_min: %g\ttick: %g\ttick_min:%g\n",  */
+/* 	  norm_tick_min, *tick, tick_min); */
+   double first_tick = ceil(axis_min /(*tick)) * (*tick);
+   double last_tick = floor(axis_max /(*tick)) * (*tick);
+   
+   *num_locations = (int)round((last_tick - first_tick)/(*tick));
+   *num_locations += 1;
+
+   double *majors = ALLOC_N_double(*num_locations);
+   for (i = 0; i < *num_locations; i++)
+      majors[i] = first_tick + (*tick) * i;
+   
+   return majors;
+}
+
+
 /* This functions fills the majors attribute of the PlotAxis object
    with the position of major ticks
 */
@@ -588,10 +682,22 @@ static void compute_major_ticks(OBJ_PTR fmkr, FM *p, PlotAxis *s, int *ierr)
       double height = ((s->vertical)? p->default_text_height_dy : p->default_text_height_dx);
       double tick_min = s->min_between_major_ticks * height;
       double tick_gap = 10.0 * height;
-      Pick_Major_Tick_Interval(fmkr, p, tick_min, tick_gap, s->length, s->log_vals, &s->interval, ierr);
-      if (*ierr != 0) return;
-      s->majors = Pick_Locations_for_Major_Ticks(s->interval, s->axis_min, s->axis_max, &s->nmajors, ierr);
-      if (*ierr != 0) return;
+
+      s->majors = (s->vincent_or_bill ? 
+		   pick_major_ticks_positions_Vincent(fmkr, p, 
+						      s->axis_min, 
+						      s->axis_max, &s->nmajors,
+						      tick_min, tick_gap, 
+						      s->length, s->log_vals, 
+						      &s->interval, ierr) :
+		   pick_major_ticks_positions_Bill(fmkr, p, s->axis_min, 
+						   s->axis_max, &s->nmajors,
+						   tick_min, tick_gap, 
+						   s->length, s->log_vals, 
+						   &s->interval, ierr));
+
+      if(*ierr || ! s->majors)
+	 return;
       s->free_majors = true;
    }
 }
@@ -626,51 +732,53 @@ static void draw_major_ticks(OBJ_PTR fmkr, FM *p, PlotAxis *s, int *ierr)
    if (did_line) axis_stroke(fmkr,p, ierr);
 }
 
-static void draw_minor_ticks(OBJ_PTR fmkr, FM *p, PlotAxis *s, int *ierr)
+static double log_subintervals[8] = {
+   0.301030, 0.477121, 0.602060, 0.698970, 
+   0.778151, 0.845098, 0.903090, 0.954243 };
+
+
+/* A function that returns a double array *TO BE FREED* containing
+   the position of the minor ticks for the given axis.
+
+   The number of ticks is stored in the cnt (long) parameter.
+
+   Returns NULL in case of problems
+*/
+static double * get_minor_ticks_location(OBJ_PTR fmkr, FM *p,
+					 PlotAxis *s, long * cnt)
 {
+   double * target = NULL;
+   int ierr = 0;
+   *cnt = 0;
+ 
+   /* First, pick up the number of ticks to be used */
    if (s->number_of_minor_intervals <= 0) {
       if (s->log_vals) {
          double interval = s->majors[1] - s->majors[0];
          s->number_of_minor_intervals = (abs(interval) != 1.0 || s->nmajors > 10)? 1 : 9; 
       }  else {
-         s->number_of_minor_intervals = Pick_Number_of_Minor_Intervals(s->interval, ierr);
-         if (*ierr != 0) return;
+         s->number_of_minor_intervals = Pick_Number_of_Minor_Intervals(s->interval, &ierr);
+         if (ierr != 0) return NULL;
       }
    }
    int i, j, nsub = s->number_of_minor_intervals;
-   double inside=0.0, outside=0.0, length;
-   bool did_line = false;
-   double log_subintervals[8];
-   log_subintervals[0] = 0.301030;
-   log_subintervals[1] = 0.477121;
-   log_subintervals[2] = 0.602060;
-   log_subintervals[3] = 0.698970;
-   log_subintervals[4] = 0.778151;
-   log_subintervals[5] = 0.845098;
-   log_subintervals[6] = 0.903090;
-   log_subintervals[7] = 0.954243;
+
    if (s->log_vals && nsub > 9) nsub = 9;
-   length = s->minor_tick_length * ((s->vertical)? p->default_text_height_dx : p->default_text_height_dy);
-   if (s->ticks_inside) inside = length;
-   if (s->ticks_outside) outside = -length;
-   if (s->top_or_right) { inside = -inside; outside = -outside; }
-   if (s->line_width != s->minor_tick_width) {
-      c_line_width_set(fmkr, p, s->line_width = s->minor_tick_width, ierr);
-      if (*ierr != 0) return;
-   }
+
+   /* First case: user-supplied position of minor ticks */
    if (s->locations_for_minor_ticks != OBJ_NIL) {
-      long cnt;
-      double *locs = Vector_Data_for_Read(s->locations_for_minor_ticks, &cnt, ierr);
-      if (*ierr != 0) return;
-      for (i=0; i < cnt; i++) {
-         if (s->vertical)
-            figure_join(fmkr, p, s->x0+inside, locs[i], s->x0+outside, locs[i], ierr);
-         else
-            figure_join(fmkr, p, locs[i], s->y0+inside, locs[i], s->y0+outside, ierr);
-         did_line = true;
-         if (*ierr != 0) return;
-      }
+      double *locs = Vector_Data_for_Read(s->locations_for_minor_ticks, 
+					  cnt, &ierr);
+      if (ierr != 0) 
+	 return NULL;
+      target = ALLOC_N_double(*cnt);
+      long i;
+      for(i = 0; i < *cnt; i++)
+	 target[i] = locs[i];
+      return target;
    } else {
+      /* We allocate sligthly more space than should be necessary: */
+      target = ALLOC_N_double(nsub * (s->nmajors + 1));
       for (i=0; i <= s->nmajors; i++) {
          double loc = (i==0)? s->majors[0] - s->interval : s->majors[i-1];
          double next_loc = (i==s->nmajors)? loc + s->interval : s->majors[i];
@@ -680,18 +788,53 @@ static void draw_minor_ticks(OBJ_PTR fmkr, FM *p, PlotAxis *s, int *ierr)
             double subloc = loc + ((!s->log_vals) ? (j * subinterval) : log_subintervals[j-1]);
             if (subloc >= next_loc) break;
             if (subloc <= s->axis_min || subloc >= s->axis_max) continue;
-            if (s->vertical) {
-               figure_join(fmkr, p, s->x0+inside, subloc, s->x0+outside, subloc, ierr);
-            }
-            else {
-               figure_join(fmkr, p, subloc, s->y0+inside, subloc, s->y0+outside, ierr);
-            }
-            did_line = true;
-            if (*ierr != 0) return;
+	    
+	    /* We add one */
+	    target[*cnt] = subloc;
+	    (*cnt)++;
          }
       }
+      return target;
    }
-   if (did_line) axis_stroke(fmkr,p, ierr);
+}
+
+
+static void draw_minor_ticks(OBJ_PTR fmkr, FM *p, PlotAxis *s, int *ierr)
+{
+   long number;
+   double * locs = get_minor_ticks_location(fmkr, p, s, &number);
+   if(! locs) {
+      *ierr = 1;
+      return ;
+   }
+   long i;
+   double inside=0.0, outside=0.0, length;
+   bool did_line = false;
+
+   /* Initialization of the various lengths */
+   length = s->minor_tick_length * ((s->vertical)? p->default_text_height_dx : p->default_text_height_dy);
+   if (s->ticks_inside) inside = length;
+   if (s->ticks_outside) outside = -length;
+   if (s->top_or_right) { inside = -inside; outside = -outside; }
+   if (s->line_width != s->minor_tick_width) {
+      c_line_width_set(fmkr, p, s->line_width = s->minor_tick_width, ierr);
+      if (*ierr != 0) return;
+   }
+
+   /* Now, we stroke the ticks */
+   for(i = 0; i < number; i++) {
+      if (s->vertical)
+	 figure_join(fmkr, p, s->x0+inside, locs[i], 
+		     s->x0+outside, locs[i], ierr);
+      else
+	 figure_join(fmkr, p, locs[i], s->y0+inside, locs[i], 
+		     s->y0+outside, ierr);
+      did_line = true;
+   }
+   /* And we free the array returned by get_minor_ticks_location */
+   free(locs);
+   if (did_line) 
+      axis_stroke(fmkr,p, ierr);
 }
 
 static void show_numeric_label(OBJ_PTR fmkr, FM *p, PlotAxis *s, 
@@ -762,7 +905,6 @@ static void free_allocated_memory(PlotAxis *s)
 }
 
 static void c_show_side(OBJ_PTR fmkr, FM *p, PlotAxis *s, int *ierr) {
-   int i;
    if (s->type == AXIS_HIDDEN) return;
    Start_Axis_Standard_State(fmkr, p, 
       s->stroke_color_R, s->stroke_color_G, s->stroke_color_B, 
@@ -1088,6 +1230,15 @@ OBJ_PTR c_axis_get_information(OBJ_PTR fmkr, FM *p, OBJ_PTR axis_spec,
    compute_major_ticks(fmkr, p, &axis, ierr);
    Hash_Set_Obj(hash, "major", Vector_New(axis.nmajors, axis.majors));
 
+   /* Then, minor ticks positions */
+   double * minor;
+   long count;
+   minor = get_minor_ticks_location(fmkr, p, &axis, &count);
+   if(minor) {
+      Hash_Set_Obj(hash, "minor", Vector_New(count, minor));
+      free(minor);
+   }
+
    /* Then, labels */
    ar = Array_New(axis.nmajors);
    axis.labels = Get_Labels(fmkr, p, &axis, ierr);
