@@ -362,7 +362,8 @@ clr_value(double n1, double n2, double hue)  // from plplot plctrl.c
 }
 
 
-static void
+// this is also used for converting images
+void
 convert_hls_to_rgb(double h, double l, double s, double *p_r, double *p_g,
                    double *p_b)  // from plplot plctrl.c
 {
@@ -510,16 +511,31 @@ c_get_color_from_colormap(OBJ_PTR fmkr, FM *p, OBJ_PTR color_map, double x,
                           int *ierr)
 {  // x is from 0 to 1.  this returns a vector for the RGB color from
    // the given colormap
-   unsigned char *buff = (unsigned char *)(String_Ptr(color_map,ierr));
-   unsigned char r, g, b, i;
-   int len = String_Len(color_map,ierr);
+   OBJ_PTR cm_len_obj;
+   OBJ_PTR lookup_obj;
+   unsigned char *buff;
+   unsigned char r, g, b;
+   int i, cm_len, lu_len;
+
+   cm_len_obj = Array_Entry(color_map, 0, ierr);
    if (*ierr != 0) RETURN_NIL;
-   if (len % 3 != 0) {
-      RAISE_ERROR("Sorry: color_map length must be a multiple of 3 "
+   cm_len = Number_to_int(cm_len_obj, ierr) + 1;
+   if (*ierr != 0) RETURN_NIL;
+   lookup_obj = Array_Entry(color_map, 1, ierr);
+   if (*ierr != 0) RETURN_NIL;
+   buff = (unsigned char *)(String_Ptr(lookup_obj,ierr));
+   if (*ierr != 0) RETURN_NIL;
+   lu_len = String_Len(lookup_obj,ierr);
+   if (*ierr != 0) RETURN_NIL;
+
+   if (3*cm_len != lu_len) {
+      RAISE_ERROR("Sorry: lookup length must be 3 times colormap length "
                   "(for R G B components)", ierr);
       RETURN_NIL;
    }
-   i = 3 * ROUND(x * ((len/3)-1));
+   // Make sure x is non-negative
+   if (x < 0.0) x = -x;
+   i = 3 * (ROUND(x * (cm_len-1)) % cm_len);
    r = buff[i]; g = buff[i+1]; b = buff[i+2];
    OBJ_PTR result = Array_New(3);
    Array_Store(result, 0, Float_New(r/255.0), ierr);
@@ -785,3 +801,43 @@ c_yaxis_stroke_color_get(OBJ_PTR fmkr, FM *p, int *ierr)
    Array_Store(result, 2, Float_New(b), ierr);
    return result;
 }
+
+
+
+void
+str_hls_to_rgb_bang(unsigned char* str, long len)
+{
+   double r,g,b,h,l,s;  
+   long n, j; 
+   // convert HLS triples to RGB triples
+   n = len/3;
+   for (j=0; j<n; j++) {
+      // 360/256 = 1.40625
+      h = str[0]*1.40625; l = str[1]/255.0; s = str[2]/255.0;
+      convert_hls_to_rgb(h,l,s,&r,&g,&b);
+      *str++ = round(r*255.0); *str++ = round(g*255.0); *str++ = round(b*255.0); 
+   }
+}
+
+
+void
+c_string_hls_to_rgb_bang(OBJ_PTR fmkr, FM *p, unsigned char* str, long len, int *ierr)
+{ str_hls_to_rgb_bang(str,len); }
+
+void
+c_string_rgb_to_hls_bang(OBJ_PTR fmkr, FM *p, unsigned char* str, long len, int *ierr)
+{
+   double r,g,b,h,l,s;  
+   long n, j; 
+   // convert RGB triples to HLS triples
+   n = len/3;
+   for (j=0; j<n; j++) {
+      // 360/256 = 1.40625
+      r = str[0]/255.0; g = str[1]/255.0; b = str[2]/255.0;
+      convert_rgb_to_hls(r, g, b, &h, &l, &s);
+      *str++ = round(h/1.40625); *str++ = round(l*255.0); *str++ = round(s*255.0); 
+   }
+}
+
+
+
